@@ -1,93 +1,27 @@
-#!/usr/bin/perl
+#!/usr/local/bin/perl
 # -*-perl-*-
-#
+
 # Modification history:
 # Written 91-12-02 through 92-01-01 by Stephen McGee.
 # Modified 92-02-11 through 92-02-22 by Chris Arthur to further generalize.
-#
-# Copyright (C) 1991-2013 Free Software Foundation, Inc.
-# This file is part of GNU Make.
-#
-# GNU Make is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 3 of the License, or (at your option) any later
-# version.
-#
-# GNU Make is distributed in the hope that it will be useful, but WITHOUT ANY
-# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
-# details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program.  If not, see <http://www.gnu.org/licenses/>.
-
+# End of modification history
 
 # Test driver routines used by a number of test suites, including
 # those for SCS, make, roll_dir, and scan_deps (?).
-#
+
 # this routine controls the whole mess; each test suite sets up a few
 # variables and then calls &toplevel, which does all the real work.
 
-# $Id$
-
-
-# The number of test categories we've run
-$categories_run = 0;
-# The number of test categroies that have passed
-$categories_passed = 0;
-# The total number of individual tests that have been run
-$total_tests_run = 0;
-# The total number of individual tests that have passed
-$total_tests_passed = 0;
-# The number of tests in this category that have been run
-$tests_run = 0;
-# The number of tests in this category that have passed
-$tests_passed = 0;
-
-
-# Yeesh.  This whole test environment is such a hack!
-$test_passed = 1;
-
-
-# Timeout in seconds.  If the test takes longer than this we'll fail it.
-$test_timeout = 5;
-
-# Path to Perl
-$perl_name = $^X;
-
-# %makeENV is the cleaned-out environment.
-%makeENV = ();
-
-# %extraENV are any extra environment variables the tests might want to set.
-# These are RESET AFTER EVERY TEST!
-%extraENV = ();
-
-# %origENV is the caller's original environment
-%origENV = %ENV;
-
-sub resetENV
-{
-  # We used to say "%ENV = ();" but this doesn't work in Perl 5.000
-  # through Perl 5.004.  It was fixed in Perl 5.004_01, but we don't
-  # want to require that here, so just delete each one individually.
-  foreach $v (keys %ENV) {
-    delete $ENV{$v};
-  }
-
-  %ENV = %makeENV;
-  foreach $v (keys %extraENV) {
-    $ENV{$v} = $extraENV{$v};
-    delete $extraENV{$v};
-  }
-}
-
 sub toplevel
 {
-  # Pull in benign variables from the user's environment
+  # Get a clean environment
 
+  %makeENV = ();
+
+  # Pull in benign variables from the user's environment
+  #
   foreach (# UNIX-specific things
-           'TZ', 'TMPDIR', 'HOME', 'USER', 'LOGNAME', 'PATH',
-           'LD_LIBRARY_PATH',
+           'TZ', 'LANG', 'TMPDIR', 'HOME', 'USER', 'LOGNAME', 'PATH',
            # Purify things
            'PURIFYOPTIONS',
            # Windows NT-specific stuff
@@ -99,15 +33,19 @@ sub toplevel
     $makeENV{$_} = $ENV{$_} if $ENV{$_};
   }
 
-  # Make sure our compares are not foiled by locale differences
-
-  $makeENV{LC_ALL} = 'C';
-
   # Replace the environment with the new one
   #
   %origENV = %ENV;
 
-  resetENV();
+  # We used to say "%ENV = ();" but this doesn't work in Perl 5.000
+  # through Perl 5.004.  It was fixed in Perl 5.004_01, but we don't
+  # want to require that here, so just delete each one individually.
+
+  foreach $v (keys %ENV) {
+    delete $ENV{$v};
+  }
+
+  %ENV = %makeENV;
 
   $| = 1;                     # unbuffered output
 
@@ -130,7 +68,7 @@ sub toplevel
 
   &parse_command_line (@ARGV);
 
-  print "OS name = '$osname'\n" if $debug;
+  print "OS name = `$osname'\n" if $debug;
 
   $workpath = "$cwdslash$workdir";
   $scriptpath = "$cwdslash$scriptdir";
@@ -165,7 +103,7 @@ sub toplevel
         $dir = $1;
         push (@rmdirs, $dir);
         -d "$workpath/$dir"
-           || mkdir ("$workpath/$dir", 0777)
+	   || mkdir ("$workpath/$dir", 0777)
            || &error ("Couldn't mkdir $workpath/$dir: $!\n");
       }
     }
@@ -174,23 +112,24 @@ sub toplevel
   {
     print "Finding tests...\n";
     opendir (SCRIPTDIR, $scriptpath)
-        || &error ("Couldn't opendir $scriptpath: $!\n");
-    @dirs = grep (!/^(\..*|CVS|RCS)$/, readdir (SCRIPTDIR) );
+	|| &error ("Couldn't opendir $scriptpath: $!\n");
+    @dirs = grep (!/^(\.\.?|CVS|RCS)$/, readdir (SCRIPTDIR) );
     closedir (SCRIPTDIR);
     foreach $dir (@dirs)
     {
-      next if ($dir =~ /^(\..*|CVS|RCS)$/ || ! -d "$scriptpath/$dir");
+      next if ($dir =~ /^\.\.?$/ || $dir eq 'CVS' || $dir eq 'RCS'
+               || ! -d "$scriptpath/$dir");
       push (@rmdirs, $dir);
       mkdir ("$workpath/$dir", 0777)
            || &error ("Couldn't mkdir $workpath/$dir: $!\n");
       opendir (SCRIPTDIR, "$scriptpath/$dir")
-          || &error ("Couldn't opendir $scriptpath/$dir: $!\n");
-      @files = grep (!/^(\..*|CVS|RCS|.*~)$/, readdir (SCRIPTDIR) );
+	  || &error ("Couldn't opendir $scriptpath/$dir: $!\n");
+      @files = grep (!/^(\.\.?|CVS|RCS)$/, readdir (SCRIPTDIR) );
       closedir (SCRIPTDIR);
       foreach $test (@files)
       {
-        -d $test and next;
-        push (@TESTS, "$dir/$test");
+        next if $test =~ /~$/ || -d $test;
+	push (@TESTS, "$dir/$test");
       }
     }
   }
@@ -211,24 +150,17 @@ sub toplevel
 
   $| = 1;
 
-  $categories_failed = $categories_run - $categories_passed;
-  $total_tests_failed = $total_tests_run - $total_tests_passed;
-
-  if ($total_tests_failed)
+  if ($num_failed)
   {
-    print "\n$total_tests_failed Test";
-    print "s" unless $total_tests_failed == 1;
-    print " in $categories_failed Categor";
-    print ($categories_failed == 1 ? "y" : "ies");
-    print " Failed (See .$diffext* files in $workdir dir for details) :-(\n\n";
+    print "\n$num_failed Test";
+    print "s" unless $num_failed == 1;
+    print " Failed (See .$diffext files in $workdir dir for details) :-(\n\n";
     return 0;
   }
   else
   {
-    print "\n$total_tests_passed Test";
-    print "s" unless $total_tests_passed == 1;
-    print " in $categories_passed Categor";
-    print ($categories_passed == 1 ? "y" : "ies");
+    print "\n$counter Test";
+    print "s" unless $counter == 1;
     print " Complete ... No Failures :-)\n\n";
     return 1;
   }
@@ -237,10 +169,9 @@ sub toplevel
 sub get_osname
 {
   # Set up an initial value.  In perl5 we can do it the easy way.
+  #
   $osname = defined($^O) ? $^O : '';
 
-  # Find a path to Perl
-
   # See if the filesystem supports long file names with multiple
   # dots.  DOS doesn't.
   $short_filenames = 0;
@@ -275,15 +206,15 @@ sub get_osname
     eval "chop (\$osname = `sh -c 'uname -nmsr 2>&1'`)";
     if ($osname =~ /not found/i)
     {
-        $osname = "(something posixy with no uname)";
+	$osname = "(something unixy with no uname)";
     }
     elsif ($@ ne "" || $?)
     {
         eval "chop (\$osname = `sh -c 'uname -a 2>&1'`)";
         if ($@ ne "" || $?)
         {
-            $osname = "(something posixy)";
-        }
+	    $osname = "(something unixy)";
+	}
     }
     $vos = 0;
     $pathsep = "/";
@@ -417,17 +348,16 @@ sub print_banner
 
 sub run_each_test
 {
-  $categories_run = 0;
+  $counter = 0;
 
   foreach $testname (sort @TESTS)
   {
-    ++$categories_run;
-    $suite_passed = 1;       # reset by test on failure
+    $counter++;
+    $test_passed = 1;       # reset by test on failure
     $num_of_logfiles = 0;
     $num_of_tmpfiles = 0;
     $description = "";
     $details = "";
-    $old_makefile = undef;
     $testname =~ s/^$scriptpath$pathsep//;
     $perl_testname = "$scriptpath$pathsep$testname";
     $testname =~ s/(\.pl|\.perl)$//;
@@ -438,19 +368,17 @@ sub run_each_test
       $logext = 'l';
       $diffext = 'd';
       $baseext = 'b';
-      $runext = 'r';
       $extext = '';
-    } else {
+   }
+    else {
       $logext = 'log';
       $diffext = 'diff';
       $baseext = 'base';
-      $runext = 'run';
       $extext = '.';
     }
     $log_filename = "$testpath.$logext";
     $diff_filename = "$testpath.$diffext";
     $base_filename = "$testpath.$baseext";
-    $run_filename = "$testpath.$runext";
     $tmp_filename = "$testpath.$tmpfilesuffix";
 
     &setup_for_test;          # suite-defined
@@ -462,52 +390,48 @@ sub run_each_test
     print $output;
 
     # Run the actual test!
-    $tests_run = 0;
-    $tests_passed = 0;
-
+    #
     $code = do $perl_testname;
-
-    $total_tests_run += $tests_run;
-    $total_tests_passed += $tests_passed;
-
-    # How did it go?
     if (!defined($code))
     {
-      $suite_passed = 0;
-      if (length ($@)) {
+      $test_passed = 0;
+      if (length ($@))
+      {
         warn "\n*** Test died ($testname): $@\n";
-      } else {
+      }
+      else
+      {
         warn "\n*** Couldn't run $perl_testname\n";
       }
     }
     elsif ($code == -1) {
-      $suite_passed = 0;
+      $test_passed = 0;
     }
     elsif ($code != 1 && $code != -1) {
-      $suite_passed = 0;
+      $test_passed = 0;
       warn "\n*** Test returned $code\n";
     }
 
-    if ($suite_passed) {
-      ++$categories_passed;
-      $status = "ok     ($tests_passed passed)";
+    if ($test_passed) {
+      $status = "ok";
       for ($i = $num_of_tmpfiles; $i; $i--)
       {
-        &rmfiles ($tmp_filename . &num_suffix ($i) );
+        &delete ($tmp_filename . &num_suffix ($i) );
       }
 
       for ($i = $num_of_logfiles ? $num_of_logfiles : 1; $i; $i--)
       {
-        &rmfiles ($log_filename . &num_suffix ($i) );
-        &rmfiles ($base_filename . &num_suffix ($i) );
+        &delete ($log_filename . &num_suffix ($i) );
+        &delete ($base_filename . &num_suffix ($i) );
       }
     }
-    elsif (!defined $code || $code > 0) {
-      $status = "FAILED ($tests_passed/$tests_run passed)";
+    elsif ($code > 0) {
+      $status = "FAILED";
+      $num_failed++;
     }
     elsif ($code < 0) {
       $status = "N/A";
-      --$categories_run;
+      --$counter;
     }
 
     # If the verbose option has been specified, then a short description
@@ -542,7 +466,7 @@ sub run_each_test
 # If the keep flag is not set, this subroutine deletes all filenames that
 # are sent to it.
 
-sub rmfiles
+sub delete
 {
   local(@files) = @_;
 
@@ -559,10 +483,12 @@ sub print_standard_usage
   local($plname,@moreusage) = @_;
   local($line);
 
-  print "usage:\t$plname [testname] [-verbose] [-detail] [-keep]\n";
-  print "\t\t\t[-profile] [-usage] [-help] [-debug]\n";
-  foreach (@moreusage) {
-    print "\t\t\t$_\n";
+  print "Usage:  perl $plname [testname] [-verbose] [-detail] [-keep]\n";
+  print "                               [-profile] [-usage] [-help] "
+      . "[-debug]\n";
+  foreach $line (@moreusage)
+  {
+    print "                               $line\n";
   }
 }
 
@@ -651,70 +577,47 @@ sub error
 sub compare_output
 {
   local($answer,$logfile) = @_;
-  local($slurp, $answer_matched) = ('', 0);
+  local($slurp);
 
-  ++$tests_run;
-
-  if (! defined $answer) {
-      print "Ignoring output ........ " if $debug;
-      $answer_matched = 1;
-  } else {
-      print "Comparing Output ........ " if $debug;
-
-      $slurp = &read_file_into_string ($logfile);
-
-      # For make, get rid of any time skew error before comparing--too bad this
-      # has to go into the "generic" driver code :-/
-      $slurp =~ s/^.*modification time .*in the future.*\n//gm;
-      $slurp =~ s/^.*Clock skew detected.*\n//gm;
-
-      if ($slurp eq $answer) {
-          $answer_matched = 1;
-      } else {
-          # See if it is a slash or CRLF problem
-          local ($answer_mod, $slurp_mod) = ($answer, $slurp);
-
-          $answer_mod =~ tr,\\,/,;
-          $answer_mod =~ s,\r\n,\n,gs;
-
-          $slurp_mod =~ tr,\\,/,;
-          $slurp_mod =~ s,\r\n,\n,gs;
-
-          $answer_matched = ($slurp_mod eq $answer_mod);
-
-          # If it still doesn't match, see if the answer might be a regex.
-          if (!$answer_matched && $answer =~ m,^/(.+)/$,) {
-              $answer_matched = ($slurp =~ /$1/);
-              if (!$answer_matched && $answer_mod =~ m,^/(.+)/$,) {
-                  $answer_matched = ($slurp_mod =~ /$1/);
-              }
-          }
-      }
+  if ($debug)
+  {
+    print "Comparing Output ........ ";
   }
 
-  if ($answer_matched && $test_passed)
+  $slurp = &read_file_into_string ($logfile);
+
+  # For make, get rid of any time skew error before comparing--too bad this
+  # has to go into the "generic" driver code :-/
+  $slurp =~ s/^.*modification time .*in the future.*\n//gm;
+  $slurp =~ s/^.*Clock skew detected.*\n//gm;
+
+  if ($slurp eq $answer)
   {
-    print "ok\n" if $debug;
-    ++$tests_passed;
+    if ($debug)
+    {
+      print "ok\n";
+    }
     return 1;
   }
-
-  if (! $answer_matched) {
-    print "DIFFERENT OUTPUT\n" if $debug;
-
+  else
+  {
+    if ($debug)
+    {
+      print "DIFFERENT OUTPUT\n";
+    }
+    $test_passed = 0;
     &create_file (&get_basefile, $answer);
-    &create_file (&get_runfile, $command_string);
-
-    print "\nCreating Difference File ...\n" if $debug;
 
+    if ($debug)
+    {
+      print "\nCreating Difference File ...\n";
+    }
     # Create the difference file
-
     local($command) = "diff -c " . &get_basefile . " " . $logfile;
     &run_command_with_output(&get_difffile,$command);
-  }
 
-  $suite_passed = 0;
-  return 0;
+    return 0;
+  }
 }
 
 sub read_file_into_string
@@ -733,9 +636,6 @@ sub read_file_into_string
   return $slurp;
 }
 
-my @OUTSTACK = ();
-my @ERRSTACK = ();
-
 sub attach_default_output
 {
   local ($filename) = @_;
@@ -748,16 +648,17 @@ sub attach_default_output
     return 1;
   }
 
-  my $dup = undef;
-  open($dup, '>&', STDOUT) or error("ado: $! duping STDOUT\n", 1);
-  push @OUTSTACK, $dup;
+  open ("SAVEDOS" . $default_output_stack_level . "out", ">&STDOUT")
+        || &error ("ado: $! duping STDOUT\n", 1);
+  open ("SAVEDOS" . $default_output_stack_level . "err", ">&STDERR")
+        || &error ("ado: $! duping STDERR\n", 1);
 
-  $dup = undef;
-  open($dup, '>&', STDERR) or error("ado: $! duping STDERR\n", 1);
-  push @ERRSTACK, $dup;
+  open (STDOUT, "> " . $filename)
+        || &error ("ado: $filename: $!\n", 1);
+  open (STDERR, ">&STDOUT")
+        || &error ("ado: $filename: $!\n", 1);
 
-  open(STDOUT, '>', $filename) or error("ado: $filename: $!\n", 1);
-  open(STDERR, ">&STDOUT") or error("ado: $filename: $!\n", 1);
+  $default_output_stack_level++;
 }
 
 # close the current stdout/stderr, and restore the previous ones from
@@ -774,47 +675,23 @@ sub detach_default_output
     return 1;
   }
 
-  @OUTSTACK or error("default output stack has flown under!\n", 1);
-
-  close(STDOUT);
-  close(STDERR);
-
-  open (STDOUT, '>&', pop @OUTSTACK) or error("ddo: $! duping STDOUT\n", 1);
-  open (STDERR, '>&', pop @ERRSTACK) or error("ddo: $! duping STDERR\n", 1);
-}
-
-# This runs a command without any debugging info.
-sub _run_command
-{
-  my $code;
-
-  # We reset this before every invocation.  On Windows I think there is only
-  # one environment, not one per process, so I think that variables set in
-  # test scripts might leak into subsequent tests if this isn't reset--???
-  resetENV();
-
-  eval {
-      my $pid = fork();
-      if (! $pid) {
-          exec(@_) or die "Cannot execute $_[0]\n";
-      }
-      local $SIG{ALRM} = sub { my $e = $ERRSTACK[0]; print $e "\nTest timed out after $test_timeout seconds\n"; die "timeout\n"; };
-      alarm $test_timeout;
-      waitpid($pid, 0) > 0 or die "No such pid: $pid\n";
-      $code = $?;
-      alarm 0;
-  };
-  if ($@) {
-      # The eval failed.  If it wasn't SIGALRM then die.
-      $@ eq "timeout\n" or die "Command failed: $@";
-
-      # Timed out.  Resend the alarm to our process group to kill the children.
-      $SIG{ALRM} = 'IGNORE';
-      kill -14, $$;
-      $code = 14;
+  if (--$default_output_stack_level < 0)
+  {
+    &error ("default output stack has flown under!\n", 1);
   }
 
-  return $code;
+  close (STDOUT);
+  close (STDERR);
+
+  open (STDOUT, ">&SAVEDOS" . $default_output_stack_level . "out")
+        || &error ("ddo: $! duping STDOUT\n", 1);
+  open (STDERR, ">&SAVEDOS" . $default_output_stack_level . "err")
+        || &error ("ddo: $! duping STDERR\n", 1);
+
+  close ("SAVEDOS" . $default_output_stack_level . "out")
+        || &error ("ddo: $! closing SCSDOSout\n", 1);
+  close ("SAVEDOS" . $default_output_stack_level . "err")
+         || &error ("ddo: $! closing SAVEDOSerr\n", 1);
 }
 
 # run one command (passed as a list of arg 0 - n), returning 0 on success
@@ -822,11 +699,17 @@ sub _run_command
 
 sub run_command
 {
-  print "\nrun_command: @_\n" if $debug;
-  my $code = _run_command(@_);
-  print "run_command returned $code.\n" if $debug;
+  local ($code);
 
-  return $code;
+  if ($debug)
+  {
+    print "\nrun_command: @_\n";
+    $code = system @_;
+    print "run_command: \"@_\" returned $code.\n";
+    return $code;
+  }
+
+  return system @_;
 }
 
 # run one command (passed as a list of arg 0 - n, with arg 0 being the
@@ -836,17 +719,16 @@ sub run_command
 
 sub run_command_with_output
 {
-  my $filename = shift;
+  local ($filename) = shift;
+  local ($code);
 
-  print "\nrun_command_with_output($filename,$runname): @_\n" if $debug;
   &attach_default_output ($filename);
-  my $code = eval { _run_command(@_) };
-  my $err = $@;
+  $code = system @_;
   &detach_default_output;
-
-  $err and die $err;
-
-  print "run_command_with_output returned $code.\n" if $debug;
+  if ($debug)
+  {
+    print "run_command_with_output: \"@_\" returned $code.\n";
+  }
 
   return $code;
 }
@@ -945,7 +827,7 @@ sub touch
 
   foreach $file (@_) {
     (open(T, ">> $file") && print(T "\n") && close(T))
-        || &error("Couldn't touch $file: $!\n", 1);
+	|| &error("Couldn't touch $file: $!\n", 1);
   }
 }
 
@@ -1215,15 +1097,6 @@ sub get_difffile
   return ($diff_filename . &num_suffix ($num_of_logfiles));
 }
 
-# This subroutine returns a command filename with a number appended
-# to the end corresponding to how many logfiles (and thus command files)
-# have been created in the current running test.
-
-sub get_runfile
-{
-  return ($run_filename . &num_suffix ($num_of_logfiles));
-}
-
 # just like logfile, only a generic tmp filename for use by the test.
 # they are automatically cleaned up unless -keep was used, or the test fails.
 # Pass an argument of 1 to return the same filename as the previous call.
